<?php 

/*
 Plugin Name: Export WooCommerce Shipping zones data
 Plugin URI: https://profiles.wordpress.org/rynald0s
 Description: This plugin exports your WooCommerce shipping zones data, for import to another site. Works for table rate shipping, too.
 Author: Rynaldo Stoltz
 Author URI: https://github.com/rynaldos
 Version: 1.0.1
 License: GPLv3 or later License
 URI: http://www.gnu.org/licenses/gpl-3.0.html
 */


class csvShippingZonesExport {
  /**
   * Constructor
   */
  public function __construct() {
    global $wpdb;
    $this->db = $wpdb; 

    if (isset($_GET['zones'])) {

      $csv_zones = $this->generate_csv_zones();
      header("Pragma: public");
      header("Expires: 0");
      header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
      header("Cache-Control: private", false);
      header("Content-Type: application/octet-stream");
      header("Content-Disposition: attachment; filename=\"report.csv\";");
      header("Content-Transfer-Encoding: binary");

      echo $csv_zones;
  }

      if (isset($_GET['zone_methods'])) {

      $csv_methods = $this->generate_csv_methods();
      header("Pragma: public");
      header("Expires: 0");
      header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
      header("Cache-Control: private", false);
      header("Content-Type: application/octet-stream");
      header("Content-Disposition: attachment; filename=\"report.csv\";");
      header("Content-Transfer-Encoding: binary");

      echo $csv_methods;

      exit;
    }

    if (isset($_GET['zone_locations'])) {

      $csv_locations = $this->generate_csv_locations();
      header("Pragma: public");
      header("Expires: 0");
      header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
      header("Cache-Control: private", false);
      header("Content-Type: application/octet-stream");
      header("Content-Disposition: attachment; filename=\"report.csv\";");
      header("Content-Transfer-Encoding: binary");

      echo $csv_locations;

      exit;
    }

    if (isset($_GET['shipping_rate_methods'])) {

      $csv_shipping_rate_methods = $this->generate_rate_methods();
      header("Pragma: public");
      header("Expires: 0");
      header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
      header("Cache-Control: private", false);
      header("Content-Type: application/octet-stream");
      header("Content-Disposition: attachment; filename=\"report.csv\";");
      header("Content-Transfer-Encoding: binary");

      echo $csv_shipping_rate_methods;

      exit;
    }

    if (isset($_GET['table_rate_methods'])) {

      $csv_table_rate_methods = $this->generate_table_rate_methods();
      header("Pragma: public");
      header("Expires: 0");
      header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
      header("Cache-Control: private", false);
      header("Content-Type: application/octet-stream");
      header("Content-Disposition: attachment; filename=\"report.csv\";");
      header("Content-Transfer-Encoding: binary");

      echo $csv_table_rate_methods;

      exit;
    }

    add_action('admin_menu', array($this, 'admin_menu'));

    add_filter('query_vars', array($this, 'query_vars'));
    add_action('parse_request', array($this, 'parse_request'));
  }

  public function admin_menu() {
    add_menu_page('Export shipping zones', 'Export shipping zones', 'manage_options', 'export_shipping_zones', array($this, 'export_shipping_zones'));
  }

  /**
   * Allow for custom query variables
   */

  public function query_vars($query_vars) {
    $query_vars[] = 'export_shipping_zones';
    return $query_vars;
  }

  /**
   * Parse the request
   */

  public function parse_request(&$wp) {
    if (array_key_exists('export_shipping_zones', $wp->query_vars)) {
      $this->export_shipping_zones();
      exit;
    }
  }

  /**
   * Export zones
   */

  public function export_shipping_zones() {
    echo '<div id="icon-tools" class="icon32">
</div>';
    echo '<h2>Export all built-in shipping zones, methods, locations, and rate settings</h2>';

    echo '<p><a href="?page=export_shipping_zones&zones" class="button">Export the Shipping zones</a></p>';
    echo '<p><a href="?page=export_shipping_zones&zone_methods" class="button">Export the Shipping zone methods</a></p>';
    echo '<p><a href="?page=export_shipping_zones&zone_locations" class="button">Export the Shipping zone locations</a></p>';
    

    echo '<p><a href="?page=export_shipping_zones&shipping_rate_methods" class="button">Export all free, local pickup, and flat rate settings and rates</a></p>';


    echo '<h2>Export Table rates shipping</h2>';

    echo '<p><a href="?page=export_shipping_zones&table_rate_methods" class="button">Export the Table rates shipping settings</a></p>';

    echo '<p>Usage: </p>';
    echo '<p>Click to export each one of the database tables you see above and it will automatically be downloaded to your PC.</p>';
    echo '<p>Upload these CSV files to your new site using the free <a href="https://wordpress.org/plugins/wp-csv-to-database/">CSV to Database</a> plugin</p>';
    echo '<p>If you are not using Table rate shipping, then there is no need to export the Table rates table. If you are using Table rates and only want to export and reimport your Table rates, then only export the Table rates tables, and reimport using the same free plugin as mentioned above.</p>';
    echo '<p>The next version will include its own built in uploader so we can keep things clean and all in one place.</p>';
  }

  /**
   * Converting data to CSV
   */

  public function generate_csv_zones() {

    ob_end_clean();
    global $wpdb;
    $table = 'wp_woocommerce_shipping_zones';
    $field       = '';
    $getField    = '';
    $result      = $wpdb->get_results( "SELECT * FROM " . $table . "" );
    $requestedTable  = $wpdb->get_results( "SHOW COLUMNS FROM " . $table .  "");

        if ( ! $requestedTable ) {
        //error occurred
        wp_die( "Can't get database columns info." );
        }

        $fieldsCount = count( $requestedTable );

        foreach ( $requestedTable as $column ) {
        $getField .= $column->Field . ',';
        }

        $sub         = substr_replace( $getField, '', -1 );
        $fields      = $sub . "\n"; // Get fields names
        $csv_file_name   = $table . '_' . date( 'Ymd_His' ) . '.csv';

        // Get fields values with last comma excluded
        foreach ( $result as $row ) {
        foreach ( $row as $data ) {
            $value   = str_replace( array( "\n", "\n\r", "\r\n", "\r" ), "\t", $data ); // Replace new line with tab
            $value   = str_getcsv( $value, ",", "\"", "\\" ); // SEQUENCING DATA IN CSV FORMAT, REQUIRED PHP >= 5.3.0
            $fields  .= $value[ 0 ] . ','; // Separate fields with comma
        }
        $fields  = substr_replace( $fields, '', -1 ); // Remove extra space at end of string
        $fields  .= "\n"; // Force new line if loop complete
        }

        //header("Content-type: text/x-csv");
        header( "Content-type: text/csv" );
        header( "Content-Transfer-Encoding: binary" );
        header( "Content-Disposition: attachment; filename=" . $csv_file_name );
        header( "Content-type: application/x-msdownload" );
        header( "Pragma: no-cache" );
        header( "Expires: 0" );

        echo $fields;
        exit;
    }

    public function generate_csv_methods() {

    ob_end_clean();
    global $wpdb;
    $table = 'wp_woocommerce_shipping_zone_methods';
    $field       = '';
    $getField    = '';
    $result      = $wpdb->get_results( "SELECT * FROM " . $table . "" );
    $requestedTable  = $wpdb->get_results( "SHOW COLUMNS FROM " . $table .  "");

        if ( ! $requestedTable ) {
        //error occurred
        wp_die( "Can't get database columns info." );
        }

        $fieldsCount = count( $requestedTable );

        foreach ( $requestedTable as $column ) {
        $getField .= $column->Field . ',';
        }

        $sub         = substr_replace( $getField, '', -1 );
        $fields      = $sub . "\n"; // Get fields names
        $csv_file_name   = $table . '_' . date( 'Ymd_His' ) . '.csv';

        // Get fields values with last comma excluded
        foreach ( $result as $row ) {
        foreach ( $row as $data ) {
            $value   = str_replace( array( "\n", "\n\r", "\r\n", "\r" ), "\t", $data ); // Replace new line with tab
            $value   = str_getcsv( $value, ",", "\"", "\\" ); // SEQUENCING DATA IN CSV FORMAT, REQUIRED PHP >= 5.3.0
            $fields  .= $value[ 0 ] . ','; // Separate fields with comma
        }
        $fields  = substr_replace( $fields, '', -1 ); // Remove extra space at end of string
        $fields  .= "\n"; // Force new line if loop complete
        }

        //header("Content-type: text/x-csv");
        header( "Content-type: text/csv" );
        header( "Content-Transfer-Encoding: binary" );
        header( "Content-Disposition: attachment; filename=" . $csv_file_name );
        header( "Content-type: application/x-msdownload" );
        header( "Pragma: no-cache" );
        header( "Expires: 0" );

        echo $fields;
        exit;
    }

    public function generate_csv_locations() {

    ob_end_clean();
    global $wpdb;
    $table = 'wp_woocommerce_shipping_zone_locations';
    $field       = '';
    $getField    = '';
    $result      = $wpdb->get_results( "SELECT * FROM " . $table . "" );
    $requestedTable  = $wpdb->get_results( "SHOW COLUMNS FROM " . $table .  "");

        if ( ! $requestedTable ) {
        //error occurred
        wp_die( "Can't get database columns info." );
        }

        $fieldsCount = count( $requestedTable );

        foreach ( $requestedTable as $column ) {
        $getField .= $column->Field . ',';
        }

        $sub         = substr_replace( $getField, '', -1 );
        $fields      = $sub . "\n"; // Get fields names
        $csv_file_name   = $table . '_' . date( 'Ymd_His' ) . '.csv';

        // Get fields values with last comma excluded
        foreach ( $result as $row ) {
        foreach ( $row as $data ) {
            $value   = str_replace( array( "\n", "\n\r", "\r\n", "\r" ), "\t", $data ); // Replace new line with tab
            $value   = str_getcsv( $value, ",", "\"", "\\" ); // SEQUENCING DATA IN CSV FORMAT, REQUIRED PHP >= 5.3.0
            $fields  .= $value[ 0 ] . ','; // Separate fields with comma
        }
        $fields  = substr_replace( $fields, '', -1 ); // Remove extra space at end of string
        $fields  .= "\n"; // Force new line if loop complete
        }

        //header("Content-type: text/x-csv");
        header( "Content-type: text/csv" );
        header( "Content-Transfer-Encoding: binary" );
        header( "Content-Disposition: attachment; filename=" . $csv_file_name );
        header( "Content-type: application/x-msdownload" );
        header( "Pragma: no-cache" );
        header( "Expires: 0" );

        echo $fields;
        exit;
    }

    public function generate_rate_methods() {

    ob_end_clean();
    global $wpdb;
    $table = 'wp_options';
    $field       = '';
    $getField    = '';
    $result      = $wpdb->get_results( "SELECT * FROM " . $table ." WHERE option_name LIKE 'woocommerce_table_rate%' OR option_name LIKE 'woocommerce_free%' OR option_name LIKE 'woocommerce_local_pickup_%' OR option_name LIKE 'woocommerce_flat_%' " );
    $requestedTable  = $wpdb->get_results( "SHOW COLUMNS FROM " . $table .  "");

        if ( ! $requestedTable ) {
        //error occurred
        wp_die( "Can't get database columns info." );
        }

        $fieldsCount = count( $requestedTable );

        foreach ( $requestedTable as $column ) {
        $getField .= $column->Field . ',';
        }

        $sub         = substr_replace( $getField, '', -1 );
        $fields      = $sub . "\n"; // Get fields names
        $csv_file_name   = $table . '_' . date( 'Ymd_His' ) . '.csv';

        // Get fields values with last comma excluded
        foreach ( $result as $row ) {
        foreach ( $row as $data ) {
            $value   = str_replace( array( "\n", "\n\r", "\r\n", "\r" ), "\t", $data ); // Replace new line with tab
            $value   = str_getcsv( $value, ",", "\"", "\\" ); // SEQUENCING DATA IN CSV FORMAT, REQUIRED PHP >= 5.3.0
            $fields  .= $value[ 0 ] . ','; // Separate fields with comma
        }
        $fields  = substr_replace( $fields, '', -1 ); // Remove extra space at end of string
        $fields  .= "\n"; // Force new line if loop complete
        }

        //header("Content-type: text/x-csv");
        header( "Content-type: text/csv" );
        header( "Content-Transfer-Encoding: binary" );
        header( "Content-Disposition: attachment; filename=" . $csv_file_name );
        header( "Content-type: application/x-msdownload" );
        header( "Pragma: no-cache" );
        header( "Expires: 0" );

        echo $fields;
        exit;
    }

    public function generate_table_rate_methods() {

    ob_end_clean();
    global $wpdb;
    $table = 'wp_woocommerce_shipping_table_rates';
    $field       = '';
    $getField    = '';
    $result      = $wpdb->get_results( "SELECT * FROM " . $table . "" );
    $requestedTable  = $wpdb->get_results( "SHOW COLUMNS FROM " . $table .  "");

        if ( ! $requestedTable ) {
        //error occurred
        wp_die( "Can't get database columns info." );
        }

        $fieldsCount = count( $requestedTable );

        foreach ( $requestedTable as $column ) {
        $getField .= $column->Field . ',';
        }

        $sub         = substr_replace( $getField, '', -1 );
        $fields      = $sub . "\n"; // Get fields names
        $csv_file_name   = $table . '_' . date( 'Ymd_His' ) . '.csv';

        // Get fields values with last comma excluded
        foreach ( $result as $row ) {
        foreach ( $row as $data ) {
            $value   = str_replace( array( "\n", "\n\r", "\r\n", "\r" ), "\t", $data ); // Replace new line with tab
            $value   = str_getcsv( $value, ",", "\"", "\\" ); // SEQUENCING DATA IN CSV FORMAT, REQUIRED PHP >= 5.3.0
            $fields  .= $value[ 0 ] . ','; // Separate fields with comma
        }
        $fields  = substr_replace( $fields, '', -1 ); // Remove extra space at end of string
        $fields  .= "\n"; // Force new line if loop complete
        }

        //header("Content-type: text/x-csv");
        header( "Content-type: text/csv" );
        header( "Content-Transfer-Encoding: binary" );
        header( "Content-Disposition: attachment; filename=" . $csv_file_name );
        header( "Content-type: application/x-msdownload" );
        header( "Pragma: no-cache" );
        header( "Expires: 0" );

        echo $fields;
        exit;
    }
}

// Instantiate a singleton of this plugin
$csvShippingZonesExport = new csvShippingZonesExport();

